#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from v2ray_util import run_type

from ..util_core.v2ray import restart
from ..util_core.utils import readchar, random_email, ColorStr
from ..util_core.group import Vmess, Socks, Mtproto, SS
from ..util_core.writer import ClientWriter, GroupWriter
from ..util_core.selector import ClientSelector, GroupSelector

@restart()
def alterid():
    cs = ClientSelector(_('modify alterId'))
    group = cs.group

    if group == None:
        pass
    else:
        client_index = cs.client_index
        if type(group.node_list[client_index]) == Vmess:
            print("{}: {}".format(_("node alterID"), group.node_list[client_index].alter_id))
            new_alterid = input(_("please input new alterId: "))
            if (new_alterid.isnumeric()):
                cw = ClientWriter(group.tag, group.index, client_index)
                cw.write_aid(int(new_alterid))
                print(_("alterId modify success!"))
                return True
            else:
                print(_("input error, please check is number"))
        else:
            print(_("only vmess protocol can modify alterId!"))

@restart()
def dyn_port():
    gs = GroupSelector(_('modify dyn_port'))
    group = gs.group

    if group == None:
        pass
    else:
        print('{}: {}'.format(_("dyn_port status"), group.dyp))
        gw = GroupWriter(group.tag, group.index)
        choice = readchar(_("open/close dyn_port(y/n): ")).lower()

        if choice == 'y':
            newAlterId = input(_("please input dyn_port alterID(default 32): "))
            newAlterId = '32' if newAlterId == '' else newAlterId
            if (newAlterId.isdecimal()):
                gw.write_dyp(True, newAlterId)
                print(_("open dyn_port success!"))
                return True
            else:
                print(_("input error, please check is number"))
        elif choice == 'n':
            gw.write_dyp(False)
            print(_("close dyn_port success!"))
            return True
        else:
            print(_("input error, please input again"))

@restart()
def new_email():
    cs = ClientSelector(_('modify email'))
    group = cs.group

    if group == None:
        pass
    elif type(group.node_list[0]) == Socks:
        print(_("Socks5 don't support email!"))
    else:
        client_index = cs.client_index
        group_list = cs.group_list
        print("{}: {}".format(_("node email"), group.node_list[client_index].user_info))
        email = ""
        while True:
            is_duplicate_email=False
            remail = random_email()
            tip = _("create random email:") + ColorStr.cyan(remail) + _(", enter to use it or input new email: ")
            email = input(tip)
            if email == "":
                email = remail
            from ..util_core.utils import is_email
            if not is_email(email):
                print(_("not email, please input again"))
                continue
            
            for loop_group in group_list:
                for node in loop_group.node_list:
                    if node.user_info == None or node.user_info == '':
                        continue
                    elif node.user_info == email:
                        print(_("have same email, please input other"))
                        is_duplicate_email = True
                        break              
            if not is_duplicate_email:
                break

        if email != "":
            cw = ClientWriter(group.tag, group.index, client_index)
            cw.write_email(email)
            print(_("modify email success!!"))
            return True

@restart()
def new_uuid():
    cs = ClientSelector(_('modify uuid'))
    group = cs.group

    if group == None:
        pass
    else:
        client_index = cs.client_index
        if type(group.node_list[client_index]) == Vmess:
            print("{}: {}".format(_("node UUID"), group.node_list[client_index].password))
            choice = readchar(_("get new UUID?(y/n): ")).lower()
            if choice == "y":
                import uuid
                new_uuid = uuid.uuid1()
                print("{}: {}".format(_("new UUID"),new_uuid))
                cw = ClientWriter(group.tag, group.index, client_index)
                cw.write_uuid(new_uuid)
                print(_("UUID modify success!"))
                return True
            else:
                print(_("undo modify"))
        else:
            print(_("only vmess protocol can modify uuid!"))

@restart(True)
def port():
    gs = GroupSelector(_('modify port'))
    group = gs.group

    if group == None:
        pass
    else:
        if group.end_port:
            port_info = "{0}-{1}".format(group.port, group.end_port)
        else:
            port_info = group.port
        print('{}: {}'.format(_("group port"), port_info))
        new_port_info = input(_("please input new port(support range port(use '-' as separator), all range port can effect):"))
        import re
        if new_port_info.isdecimal() or re.match(r'^\d+\-\d+$', new_port_info):
            gw = GroupWriter(group.tag, group.index)
            gw.write_port(new_port_info)
            print(_('port modify success!'))
            return True
        else:
            print(_("input error!"))

@restart()
def tfo():
    gs = GroupSelector(_('modify tcpFastOpen'))
    group = gs.group

    if group == None:
        pass
    else:
        if type(group.node_list[0]) == Mtproto or type(group.node_list[0]) == SS:
            print(_("{} MTProto/Shadowsocks don't support tcpFastOpen!!!".format(run_type.capitalize())))
            print("")
            return
        
        print('{}: {}'.format(_("group tcpFastOpen"), group.tfo))
        print("")
        print(_("1.open TFO(force open)"))
        print(_("2.close TFO(force close)"))
        print(_("3.delete TFO(use system default profile)"))
        choice = readchar(_("please select: "))
        if not choice:
            return
        if not choice in ("1", "2", "3"):
            print(_("input error, please input again"))
            return
        
        gw = GroupWriter(group.tag, group.index)
        if choice == "1":
            gw.write_tfo('on')
        elif choice == "2":
            gw.write_tfo('off')
        elif choice == "3":
            gw.write_tfo('del')

        return True


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import socket

from .tls import TLSModifier
from ..util_core.v2ray import restart
from ..util_core.loader import Loader
from ..util_core.writer import StreamWriter, NodeWriter
from ..util_core.utils import StreamType, ColorStr, get_ip, loop_input_choice_number, check_ip, is_ipv4, port_is_use

# https://support.cloudflare.com/hc/en-us/articles/200169156-Identifying-network-ports-compatible-with-Cloudflare-s-proxy
class CDNModifier:
    def __init__(self, domain='', cType=0):
        '''
        cType 0: vmess_ws, 1: vless_ws
        '''
        self.domain = domain
        self.cType = cType          

    @restart(True)
    def open(self, port=443):
        '''
        cloudflare cdn proxy https port(443, 2053, 2083, 2087, 2096, 8443)
        '''
        nw = NodeWriter()
        nw.create_new_port(int(port))
        reload_data = Loader()
        new_group_list = reload_data.profile.group_list
        group = new_group_list[-1]
        TLSModifier(group.tag, group.index, self.domain).turn_on(False)
        if self.cType == 0:
            StreamWriter(group.tag, group.index, StreamType.WS).write()
        elif self.cType == 1:
            StreamWriter(group.tag, group.index, StreamType.VLESS_WS).write()
        return True

def modify():
    port_choice = ""

    https_list=(443, 2053, 2083, 2087, 2096, 8443)

    cdn_protocol_list=('vmess_ws', 'vless_ws')

    domain = input(_("please input run cdn mode domain: "))
    if not domain:
        print(ColorStr.yellow(_("domain is empty!")))
        return

    local_ip = get_ip()
    try:
        if is_ipv4(local_ip):
            input_ip = socket.gethostbyname(domain)
        else:
            input_ip = socket.getaddrinfo(domain, None, socket.AF_INET6)[0][4][0]
    except Exception:
        print(_("domain check error!!!"))
        print("")
        return
    
    print("")
    print(_("local vps ip address: ") + local_ip + "\n")

    if input_ip != local_ip:
        print(_("domain can't analysis to local ip!!!"))
        print(_("must be close cdn proxy!"))
        print("")
        return

    for index, text in enumerate(https_list): 
        print("{}.{}".format(index + 1, text))
    port_choice = loop_input_choice_number(_("please select https port to cdn: "), len(https_list))
    if not port_choice:
        return
    new_port = https_list[port_choice - 1]
    if port_is_use(new_port):
        print("{} port is use!".format(new_port))
        return
    print("")
    for index, text in enumerate(cdn_protocol_list): 
        print("{}.{}".format(index + 1, text))
    cdn_choice = loop_input_choice_number(_("please select protocol to cdn: "), len(cdn_protocol_list))
    if not cdn_choice:
        return
    CDNModifier(domain, cdn_choice - 1).open(new_port)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import json

from .config import Config
from .group import Vmess, Vless, Socks, SS, Mtproto, Trojan
from .selector import ClientSelector

class ClientWriter:
    def __init__(self, group, client_index):
        self.config_factory = Config()
        with open(self.config_factory.get_path('config_path'), 'r') as json_file:
            self.config = json.load(json_file)
        
        self.write_path = self.config_factory.get_path("write_client_path")
        self.template_path = self.config_factory.json_path
        self.group = group
        self.client_index = client_index
        self.node = group.node_list[client_index]

    def load_template(self, template_name):
        '''
        load special template
        '''
        with open(self.template_path + "/" + template_name, 'r') as stream_file:
            template = json.load(stream_file)
        return template

    def transform(self):
        user_json = None
        if type(self.node) == Vmess:
            self.client_config = self.load_template('client.json')
            user_json = self.client_config["outbounds"][0]["settings"]["vnext"][0]
            user_json["users"][0]["id"] = self.node.password
            user_json["users"][0]["alterId"] = self.node.alter_id

        elif type(self.node) == Vless:
            self.client_config = self.load_template('client.json')
            user_json = self.client_config["outbounds"][0]["settings"]["vnext"][0]
            user_json["users"][0]["id"] = self.node.password
            del user_json["users"][0]["alterId"]
            del user_json["users"][0]["security"]
            user_json["users"][0]["encryption"] = self.node.encryption
            if self.node.flow:
                user_json["users"][0]["flow"] = self.node.flow
            self.client_config["outbounds"][0]["protocol"] = "vless" 

        elif type(self.node) == Socks:
            self.client_config = self.load_template('client_socks.json')
            user_json = self.client_config["outbounds"][0]["settings"]["servers"][0]
            user_json["users"][0]["user"] = self.node.user_info
            user_json["users"][0]["pass"] = self.node.password

        elif type(self.node) == SS:
            self.client_config = self.load_template('client_ss.json')
            user_json = self.client_config["outbounds"][0]["settings"]["servers"][0]
            user_json["method"] = self.node.method
            user_json["password"] = self.node.password

        elif type(self.node) == Trojan:
            self.client_config = self.load_template('client_trojan.json')
            user_json = self.client_config["outbounds"][0]["settings"]["servers"][0]
            user_json["password"] = self.node.password

        elif type(self.node) == Mtproto:
            print("")
            print(_("MTProto protocol only use Telegram, and can't generate client json!"))
            print("")
            exit(-1)

        user_json["port"] = int(self.group.port)
        user_json["address"] = self.group.ip

        if type(self.node) != SS:
            self.client_config["outbounds"][0]["streamSettings"] = self.config["inbounds"][self.group.index]["streamSettings"]

        if self.group.tls == 'tls':
            self.client_config["outbounds"][0]["streamSettings"]["tlsSettings"] = {}
        elif self.group.tls == 'xtls':
            self.client_config["outbounds"][0]["streamSettings"]["xtlsSettings"]["serverName"] = self.group.ip
            del self.client_config["outbounds"][0]["streamSettings"]["xtlsSettings"]["certificates"]
            del self.client_config["outbounds"][0]["streamSettings"]["xtlsSettings"]["alpn"]
            del self.client_config["outbounds"][0]["mux"]

    def write(self):
        '''
        写客户端配置文件函数
        '''
        json_dump = json.dumps(self.client_config,indent=1)
        with open(self.write_path, 'w') as write_json_file:
            write_json_file.writelines(json_dump)

        print("{0}({1})\n".format(_("save json success!"), self.write_path))

def generate():
    cs = ClientSelector(_('generate client json'))
    if not hasattr(cs, 'client_index'):
        return
    client_index = cs.client_index
    group = cs.group

    if group == None:
        pass
    else:
        cw = ClientWriter(group, client_index)
        cw.transform()
        cw.write()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import sys
import subprocess

from v2ray_util import run_type
from .util_core.v2ray import V2ray
from .util_core.utils import ColorStr, open_port, loop_input_choice_number
from .global_setting import stats_ctr, iptables_ctr, ban_bt, update_timer
from .config_modify import base, multiple, ss, stream, tls, cdn

def help():
    exec_name = sys.argv[0]
    from .util_core.config import Config
    lang = Config().get_data('lang')
    if lang == 'zh':
        print("""
{0} [-h|help] [options]
    -h, help             查看帮助
    -v, version          查看版本号
    start                启动 {bin}
    stop                 停止 {bin}
    restart              重启 {bin}
    status               查看 {bin} 运行状态
    new                  重建新的{bin} json配置文件
    update               更新 {bin} 到最新Release版本
    update [version]     更新 {bin} 到指定版本
    update.sh            更新 multi-v2ray 到最新版本
    add                  新增端口组
    add [protocol]       新增一种协议的组, 端口随机, 如 {bin} add utp 为新增utp协议
    del                  删除端口组
    info                 查看配置
    port                 修改端口
    tls                  修改tls
    tfo                  修改tcpFastOpen
    stream               修改传输协议
    cdn                  走cdn
    stats                {bin}流量统计
    iptables             iptables流量统计
    clean                清理日志
    log                  查看日志
    rm                   卸载{bin}
        """.format(exec_name[exec_name.rfind("/") + 1:], bin=run_type))
    else:
        print("""
{0} [-h|help] [options]
    -h, help             get help
    -v, version          get version
    start                start {bin}
    stop                 stop {bin}
    restart              restart {bin}
    status               check {bin} status
    new                  create new json profile
    update               update {bin} to latest
    update [version]     update {bin} to special version
    update.sh            update multi-v2ray to latest
    add                  add new group
    add [protocol]       create special protocol, random new port
    del                  delete port group
    info                 check {bin} profile
    port                 modify port
    tls                  modify tls
    tfo                  modify tcpFastOpen
    stream               modify protocol
    cdn                  cdn mode
    stats                {bin} traffic statistics
    iptables             iptables traffic statistics
    clean                clean {bin} log
    log                  check {bin} log
    rm                   uninstall {bin}
        """.format(exec_name[exec_name.rfind("/") + 1:], bin=run_type))

def updateSh():
    if os.path.exists("/.dockerenv"):
        subprocess.Popen("pip install -U v2ray_util", shell=True).wait()
    else:
        subprocess.Popen("curl -Ls https://multi.netlify.app/v2ray.sh -o temp.sh", shell=True).wait()
        subprocess.Popen("bash temp.sh -k && rm -f temp.sh", shell=True).wait()

def parse_arg():
    if len(sys.argv) == 1:
        return
    elif len(sys.argv) == 2:
        if sys.argv[1] == "start":
            V2ray.start()
        elif sys.argv[1] == "stop":
            V2ray.stop()
        elif sys.argv[1] == "restart":
            V2ray.restart()
        elif sys.argv[1] in ("-h", "help"):
            help()
        elif sys.argv[1] in ("-v", "version"):
            V2ray.version()
        elif sys.argv[1] == "status":
            V2ray.status()
        elif sys.argv[1] == "info":
            V2ray.info()
        elif sys.argv[1] == "port":
            base.port()
        elif sys.argv[1] == "tls":
            tls.modify()
        elif sys.argv[1] == "tfo":
            base.tfo()
        elif sys.argv[1] == "stream":
            stream.modify()
        elif sys.argv[1] == "stats":
            stats_ctr.manage()
        elif sys.argv[1] == "iptables":
            iptables_ctr.manage()
        elif sys.argv[1] == "clean":
            V2ray.cleanLog()
        elif sys.argv[1] == "del":
            multiple.del_port()
        elif sys.argv[1] == "add":
            multiple.new_port()
        elif sys.argv[1] == "update":
            V2ray.update()
        elif sys.argv[1] == "update.sh":
            updateSh()
        elif sys.argv[1] == "new":
            V2ray.new()
        elif sys.argv[1] == "log":
            V2ray.log()
        elif sys.argv[1] == "cdn":
            cdn.modify()
        elif sys.argv[1] == "rm":
            V2ray.remove()
    else:
        if sys.argv[1] == "add":
            multiple.new_port(sys.argv[2])
        elif sys.argv[1] == "update":
            V2ray.update(sys.argv[2])
        elif sys.argv[1] == "log":
            if sys.argv[2] in ("error", "e"):
                V2ray.log(True)
            elif sys.argv[2] in ("access", "a"):
                V2ray.log()
    sys.exit(0)

def service_manage():
    show_text = (_("start {}".format(run_type)), _("stop {}".format(run_type)), _("restart {}".format(run_type)), _("{} status".format(run_type)), _("{} log".format(run_type)))
    print("")
    for index, text in enumerate(show_text): 
        print("{}.{}".format(index + 1, text))
    choice = loop_input_choice_number(_("please select: "), len(show_text))
    if choice == 1:
        V2ray.start()
    elif choice == 2:
        V2ray.stop()
    elif choice == 3:
        V2ray.restart()
    elif choice == 4:
        V2ray.status()
    elif choice == 5:
        V2ray.log()

def user_manage():
    show_text = (_("add user"), _("add port"), _("del user"), _("del port"))
    print("")
    for index, text in enumerate(show_text): 
        print("{}.{}".format(index + 1, text))
    choice = loop_input_choice_number(_("please select: "), len(show_text))
    if not choice:
        return
    elif choice == 1:
        multiple.new_user()
    elif choice == 2:
        multiple.new_port()
        open_port()
    elif choice == 3:
        multiple.del_user()
    elif choice == 4:
        multiple.del_port()

def profile_alter():
    show_text = (_("modify email"), _("modify UUID"), _("modify alterID"), _("modify port"), _("modify stream"), _("modify tls"), 
                _("modify tcpFastOpen"), _("modify dyn_port"), _("modify shadowsocks method"), _("modify shadowsocks password"), _("CDN mode(need domain)"))
    print("")
    for index, text in enumerate(show_text): 
        print("{}.{}".format(index + 1, text))
    choice = loop_input_choice_number(_("please select: "), len(show_text))
    if not choice:
        return
    elif choice == 1:
        base.new_email()
    elif choice == 2:
        base.new_uuid()
    elif choice == 3:
        base.alterid()
    elif choice == 4:
        base.port()
    elif choice == 5:
        stream.modify()
    elif choice == 6:
        tls.modify()
    elif choice == 7:
        base.tfo()
    elif choice == 8:
        base.dyn_port()
    elif choice == 9:
        ss.modify('method')
    elif choice == 10:
        ss.modify('password')
    elif choice == 11:
        cdn.modify()

def global_setting():
    show_text = (_("{} Traffic Statistics".format(run_type.capitalize())), _("Iptables Traffic Statistics"), _("Ban Bittorrent"), _("Schedule Update {}".format(run_type.capitalize())), _("Clean {} Log".format(run_type.capitalize())), _("Change Language"))
    print("")
    for index, text in enumerate(show_text): 
        print("{}.{}".format(index + 1, text))
    choice = loop_input_choice_number(_("please select: "), len(show_text))
    if choice == 1:
        stats_ctr.manage()
    elif choice == 2:
        iptables_ctr.manage()
    elif choice == 3:
        ban_bt.manage()
    elif choice == 4:
        update_timer.manage()
    elif choice == 5:
        V2ray.cleanLog()
    elif choice == 6:
        from .util_core.config import Config
        config = Config()
        lang = config.get_data("lang")
        config.set_data("lang", "zh" if lang == "en" else "en")
        print(ColorStr.yellow(_("please run again to become effective!")))
        sys.exit(0)

def menu():
    V2ray.check()
    parse_arg()
    while True:
        print("")
        print(ColorStr.cyan(_("Welcome to {} manager".format(run_type))))
        print("")
        show_text = (_("1.{} Manage".format(run_type.capitalize())), _("2.Group Manage"), _("3.Modify Config"), _("4.Check Config"), _("5.Global Setting"), _("6.Update {}".format(run_type.capitalize())), _("7.Generate Client Json"))
        for index, text in enumerate(show_text): 
            if index % 2 == 0:
                print('{:<20}'.format(text), end="")   
            else:
                print(text)
                print("")
        print("")
        choice = loop_input_choice_number(_("please select: "), len(show_text))
        if choice == 1:
            service_manage()
        elif choice == 2:
            user_manage()
        elif choice == 3:
            profile_alter()
        elif choice == 4:
            V2ray.info()
        elif choice == 5:
            global_setting()
        elif choice == 6:
            V2ray.update()
        elif choice == 7:
            from .util_core import client
            client.generate()
        else:
            break

if __name__ == "__main__":
    menu()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import random
import string

from ..util_core.v2ray import restart
from ..util_core.loader import Loader
from ..util_core.writer import NodeWriter
from ..util_core.group import Vmess, Socks, Mtproto, SS, Vless, Trojan
from ..util_core.selector import GroupSelector, ClientSelector, CommonSelector
from ..util_core.utils import is_email, clean_iptables, random_email, ColorStr, readchar, random_port, port_is_use, xtls_flow, StreamType

@restart(True)
def new_port(new_stream=None):
    if new_stream != None and new_stream not in StreamType._value2member_map_:
        print(ColorStr.red("{} not support!".format(new_stream)))
        return
    new_port = ""
    while True:
        new_random_port = random_port(1000, 65535)
        new_port = input("{0} {1}, {2}: ".format(_("random generate port"), ColorStr.green(str(new_random_port)), _("enter to use, or input customize port")))

        if not new_port:
            new_port = str(new_random_port)
        else:
            if not new_port.isnumeric():
                print(_("input error, please input again"))
                continue
            elif port_is_use(new_port):
                print(_("port is use, please input other port!"))
                continue
        break

    print("")
    print("{}: {}".format(_("new port"), new_port))
    print("")
    nw = NodeWriter()
    nw.create_new_port(int(new_port))

    reload_data = Loader()
    new_group_list = reload_data.profile.group_list
    from .stream import modify
    modify(new_group_list[-1], new_stream)
    return True

@restart()
def new_user():
    gs = GroupSelector(_('add user'))
    group = gs.group
    group_list = gs.group_list

    if group == None:
        pass
    else:
        email = ""
        if type(group.node_list[0]) in (Vmess, Vless, Trojan): 
            while True:
                is_duplicate_email=False
                remail = random_email()
                tip = _("create random email:") + ColorStr.cyan(remail) + _(", enter to use it or input new email: ")
                email = input(tip)
                if email == "":
                    email = remail
                if not is_email(email):
                    print(_("not email, please input again"))
                    continue
                
                for loop_group in group_list:
                    for node in loop_group.node_list:
                        if node.user_info == None or node.user_info == '':
                            continue
                        elif node.user_info == email:
                            print(_("have same email, please input other"))
                            is_duplicate_email = True
                            break              
                if not is_duplicate_email:
                    break

            nw = NodeWriter(group.tag, group.index)
            info = {'email': email}
            if type(group.node_list[0]) == Trojan:
                random_pass = ''.join(random.sample(string.digits + string.ascii_letters, 8))
                tip = _("create random trojan user password:") + ColorStr.cyan(random_pass) + _(", enter to use or input new password: ")
                password = input(tip)
                if password == "":
                    password = random_pass
                info['password'] = password
            elif type(group.node_list[0]) == Vless and group.tls == "xtls":
                flow_list = xtls_flow()
                print("")
                flow = CommonSelector(flow_list, _("please select xtls flow type: ")).select()
                info['flow'] = flow
            nw.create_new_user(**info)
            return True

        elif type(group.node_list[0]) == Socks:
            print(_("local group is socks, please input user and password to create user"))
            print("")
            user = input(_("please input socks user: "))
            password = input(_("please input socks password: "))
            if user == "" or password == "":
                print(_("socks user or password is null!!"))
                exit(-1)
            info = {"user":user, "pass": password}
            nw = NodeWriter(group.tag, group.index)
            nw.create_new_user(**info)
            return True

        elif type(group.node_list[0]) == Mtproto:
            print("")
            print(_("Mtproto protocol only support one user!!"))

        elif type(group.node_list[0]) == SS:
            print("")
            print(_("Shadowsocks protocol only support one user, u can add new port to multiple SS!"))

@restart()
def del_port():
    gs = GroupSelector(_('del port'))
    group = gs.group

    if group == None:
        pass
    else:
        print(_("del group info: "))
        print(group)
        choice = readchar(_("delete?(y/n): ")).lower()
        if choice == 'y':
            nw = NodeWriter()
            nw.del_port(group)
            clean_iptables(group.port)
            return True
        else:
            print(_("undo delete"))

@restart()
def del_user():
    cs = ClientSelector(_('del user'))
    group = cs.group

    if group == None:
        pass
    else:
        client_index = cs.client_index
        print(_("del user info:"))
        print(group.show_node(client_index))
        choice = readchar(_("delete?(y/n): ")).lower()
        if choice == 'y':
            if len(group.node_list) == 1:
                clean_iptables(group.port)
            nw = NodeWriter()
            nw.del_user(group, client_index)
            return True
        else:
            print(_("undo delete"))


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import socket
import os

from ..util_core.v2ray import restart, V2ray
from ..util_core.writer import GroupWriter
from ..util_core.group import Mtproto, SS
from ..util_core.selector import GroupSelector
from ..util_core.utils import get_ip, gen_cert, readchar, is_ipv4

class TLSModifier:
    def __init__(self, group_tag, group_index, domain='', alpn=None, xtls=False):
        self.domain = domain
        self.alpn = alpn
        self.xtls = xtls
        self.writer = GroupWriter(group_tag, group_index)
    
    @restart(True)
    def turn_on(self, need_restart=True):
        print("")
        print(_("1. Let's Encrypt certificate(auto create, please prepare domain)"))
        print(_("2. Customize certificate(prepare certificate file paths)"))
        print("")
        choice = readchar(_("please select: "))
        input_domain = self.domain
        if choice == "1":
            if not input_domain:
                local_ip = get_ip()
                input_domain = input(_("please input your vps domain: "))
                try:
                    if is_ipv4(local_ip):
                        socket.gethostbyname(input_domain)
                    else:
                        socket.getaddrinfo(input_domain, None, socket.AF_INET6)[0][4][0]
                except Exception:
                    print(_("domain check error!!!"))
                    print("")
                    return

            print("")
            print(_("auto generate SSL certificate, please wait.."))
            V2ray.stop()
            gen_cert(input_domain)
            crt_file = "/root/.acme.sh/" + input_domain +"_ecc"+ "/fullchain.cer"
            key_file = "/root/.acme.sh/" + input_domain +"_ecc"+ "/"+ input_domain +".key"

            self.writer.write_tls(True, crt_file=crt_file, key_file=key_file, domain=input_domain, alpn=self.alpn, xtls=self.xtls)

        elif choice == "2":
            crt_file = input(_("please input certificate cert file path: "))
            key_file = input(_("please input certificate key file path: "))
            if not os.path.exists(crt_file) or not os.path.exists(key_file):
                print(_("certificate cert or key not exist!"))
                return
            if not input_domain:
                input_domain = input(_("please input the certificate cert file domain: "))
                if not input_domain:
                    print(_("domain is null!"))
                    return
            self.writer.write_tls(True, crt_file=crt_file, key_file=key_file, domain=input_domain, alpn=self.alpn, xtls=self.xtls)
        else:
            print(_("input error!"))
            return
        return need_restart

    @restart()
    def turn_off(self):
        self.writer.write_tls(False)
        return True

def modify():
    gs = GroupSelector(_('modify tls'))
    group = gs.group

    if group == None:
        pass
    else:
        if type(group.node_list[0]) == Mtproto or type(group.node_list[0]) == SS:
            print(_("MTProto/Shadowsocks protocol not support https!!!"))
            print("")
            return
        tm = TLSModifier(group.tag, group.index)
        tls_status = 'open' if group.tls == 'tls' else 'close'
        print("{}: {}\n".format(_("group tls status"), tls_status))
        print("")
        print(_("1.open TLS"))
        print(_("2.close TLS"))
        choice = readchar(_("please select: "))
        if not choice:
            return
        if not choice in ("1", "2"):
            print(_("input error, please input again"))
            return

        if choice == '1':
            tm.turn_on()
        elif choice == '2':
            tm.turn_off()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import sys
import uuid
import time
import subprocess
import pkg_resources
from functools import wraps
from v2ray_util import run_type
from .utils import ColorStr, open_port, get_ip, is_ipv6, random_port

def restart(port_open=False):
    """
    运行函数/方法后重启v2ray的装饰器
    """  
    def decorate(func):
        @wraps(func)
        def wrapper(*args, **kw):
            result = func(*args, **kw)
            if port_open:
                open_port()
            if result:
                V2ray.restart()
        return wrapper
    return decorate

class V2ray:

    @staticmethod
    def docker_run(command, keyword):
        subprocess.run(command, shell=True)
        print("{}ing {}...".format(keyword, run_type))
        time.sleep(1)
        if V2ray.docker_status() or keyword == "stop":
            print(ColorStr.green("{} {} success !".format(run_type, keyword)))
        else:
            print(ColorStr.red("{} {} fail !".format(run_type, keyword)))

    @staticmethod
    def run(command, keyword):
        try:
            subprocess.check_output(command, shell=True)
            print("{}ing {}...".format(keyword, run_type))
            time.sleep(2)
            if subprocess.check_output("systemctl is-active {}|grep active".format(run_type), shell=True) or keyword == "stop":
                print(ColorStr.green("{} {} success !".format(run_type, keyword)))
            else:
                raise subprocess.CalledProcessError
        except subprocess.CalledProcessError:
            print(ColorStr.red("{} {} fail !".format(run_type, keyword)))

    @staticmethod
    def docker_status():
        is_running = True
        failed = bytes.decode(subprocess.run('cat /.run.log|grep failed', shell=True, stdout=subprocess.PIPE).stdout)
        running = bytes.decode(subprocess.run('ps aux|grep /etc/{}/config.json'.format(run_type), shell=True, stdout=subprocess.PIPE).stdout)
        if failed or "/usr/bin/{bin}/{bin}".format(bin=run_type) not in running:
            is_running = False
        return is_running

    @staticmethod
    def status():
        if os.path.exists("/.dockerenv"):
            if V2ray.docker_status():
                print(ColorStr.green("{} running..".format(run_type)))
            else:
                print(bytes.decode(subprocess.run('cat /.run.log', shell=True, stdout=subprocess.PIPE).stdout))
                print(ColorStr.yellow("{} stoped..".format(run_type)))
        else:
            subprocess.call("systemctl status {}".format(run_type), shell=True)

    @staticmethod
    def version():
        v2ray_version = bytes.decode(subprocess.check_output("/usr/bin/{bin}/{bin}".format(bin=run_type) + " -version | head -n 1 | awk '{print $2}'", shell=True))
        import v2ray_util
        print("{}: {}".format(run_type, ColorStr.green(v2ray_version)))
        print("v2ray_util: {}".format(ColorStr.green(v2ray_util.__version__)))    

    @staticmethod
    def info():
        from .loader import Loader 
        print(Loader().profile)

    @staticmethod
    def update(version=None):
        if is_ipv6(get_ip()):
            print(ColorStr.yellow(_("ipv6 network not support update {soft} online, please manual donwload {soft} to update!".format(soft=run_type))))
            if run_type == "xray":
                print(ColorStr.fuchsia(_("download Xray-linux-xx.zip and run 'bash <(curl -L -s https://multi.netlify.app/go.sh) -l Xray-linux-xx.zip -x' to update")))
            else:
                print(ColorStr.fuchsia(_("download v2ray-linux-xx.zip and run 'bash <(curl -L -s https://multi.netlify.app/go.sh) -l v2ray-linux-xx.zip' to update")))
            sys.exit(0)
        if os.path.exists("/.dockerenv"):
            V2ray.stop()
        subprocess.Popen("curl -Ls https://multi.netlify.app/go.sh -o temp.sh", shell=True).wait()
        subprocess.Popen("bash temp.sh {} {} && rm -f temp.sh".format("-x" if run_type == "xray" else "", "--version {}".format(version) if version else ""), shell=True).wait()
        if os.path.exists("/.dockerenv"):
            V2ray.start()

    @staticmethod
    def cleanLog():
        subprocess.call("cat /dev/null > /var/log/{}/access.log".format(run_type), shell=True)
        subprocess.call("cat /dev/null > /var/log/{}/error.log".format(run_type), shell=True)
        print(ColorStr.green(_("clean {} log success!".format(run_type))))
        print("")

    @staticmethod
    def log(error_log=False):
        f = subprocess.Popen(['tail','-f', '-n', '100', '/var/log/{}/{}.log'.format(run_type, "error" if error_log else "access")],
                stdout=subprocess.PIPE,stderr=subprocess.PIPE)
        try:
            while True:
                print(bytes.decode(f.stdout.readline().strip()))
        except BaseException:
            print()

    @classmethod
    def restart(cls):
        if os.path.exists("/.dockerenv"):
            V2ray.stop()
            V2ray.start()
        else:
            cls.run("systemctl restart {}".format(run_type), "restart")

    @classmethod
    def start(cls):
        if os.path.exists("/.dockerenv"):
            cls.docker_run("/usr/bin/{bin}/{bin} -config=/etc/{bin}/config.json > /.run.log &".format(bin=run_type), "start")
        else:
            cls.run("systemctl start {}".format(run_type), "start")

    @classmethod
    def stop(cls):
        if os.path.exists("/.dockerenv"):
            cls.docker_run("ps aux|grep /usr/bin/{bin}/{bin}".format(bin=run_type) + "|awk '{print $1}'|xargs  -r kill -9 2>/dev/null", "stop")
        else:
            cls.run("systemctl stop {}".format(run_type), "stop")

    @classmethod
    def check(cls):
        if not os.path.exists("/etc/v2ray_util/util.cfg"):
            subprocess.call("mkdir -p /etc/v2ray_util && cp -f {} /etc/v2ray_util/".format(pkg_resources.resource_filename(__name__, 'util.cfg')), shell=True)
        if not os.path.exists("/usr/bin/{bin}/{bin}".format(bin=run_type)):
            print(ColorStr.yellow(_("check {soft} no install, auto install {soft}..".format(soft=run_type))))
            cls.update()
            cls.new()

    @classmethod
    def remove(cls):
        if os.path.exists("/.dockerenv"):
            print(ColorStr.yellow("docker run don't support remove {}!".format(run_type)))
            return
        cls.stop()
        subprocess.call("systemctl disable {}.service".format(run_type), shell=True)
        subprocess.call("rm -rf /usr/bin/{bin} /etc/systemd/system/{bin}.service".format(bin=run_type), shell=True)
        print(ColorStr.green("Removed {} successfully.".format(run_type)))
        print(ColorStr.blue("If necessary, please remove configuration file and log file manually."))

    @classmethod
    def new(cls):
        subprocess.call("rm -rf /etc/{soft}/config.json && cp {package_path}/server.json /etc/{soft}/config.json".format(soft=run_type, package_path=pkg_resources.resource_filename('v2ray_util', "json_template")), shell=True)
        new_uuid = uuid.uuid1()
        print("new UUID: {}".format(ColorStr.green(str(new_uuid))))
        new_port = random_port(1000, 65535)
        print("new port: {}".format(ColorStr.green(str(new_port))))
        subprocess.call("sed -i \"s/cc4f8d5b-967b-4557-a4b6-bde92965bc27/{uuid}/g\" /etc/{soft}/config.json && sed -i \"s/999999999/{port}/g\" /etc/{soft}/config.json".format(uuid=new_uuid, port=new_port, soft=run_type), shell=True)
        if run_type == "xray":
            subprocess.call("sed -i \"s/v2ray/xray/g\" /etc/xray/config.json", shell=True)
        from ..config_modify import stream
        stream.StreamModifier().random_kcp()
        open_port()
        cls.restart()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import pkg_resources
import configparser
from v2ray_util import run_type

CONF_FILE = '/etc/v2ray_util/util.cfg'
DATA_FILE = 'util.dat' if run_type == "v2ray" else 'xray.dat'

class Config:

    def __init__(self):
        self.config = configparser.ConfigParser()
        self.config_path = CONF_FILE
        self.data_path = pkg_resources.resource_filename('v2ray_util', DATA_FILE)
        self.json_path = pkg_resources.resource_filename('v2ray_util', "json_template")
        self.config.read(self.config_path)

    def get_path(self, key):
        if key == 'config_path' and run_type == 'xray':
            return '/etc/xray/config.json'
        return self.config.get('path', key)

    def get_data(self, key):
        return self.config.get('data', key)

    def set_data(self, key, value):
        self.config.set('data', key, value)
        self.config.write(open(self.config_path, "w"))


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import re
import subprocess

from v2ray_util import run_type
from ..util_core.v2ray import V2ray
from ..util_core.loader import Loader
from ..util_core.writer import GlobalWriter
from ..util_core.utils import bytes_2_human_readable, ColorStr, readchar

class StatsFactory:
    def __init__(self, door_port):
        self.door_port = door_port
        self.downlink_value = 0
        self.uplink_value = 0

    def __run_command(self, command):
        value = 0
        result = bytes.decode(subprocess.run(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.DEVNULL).stdout.strip())
        result_list = re.findall(r"\s+\d+\s+", result)
        if "v2ctl" not in result and len(result_list) == 1:
            value = int(result_list[0])
        return value

    def get_stats(self, meta_info, is_reset=False, is_group=False):
        is_reset = "true" if is_reset else "false"
        type_tag = ("inbound" if is_group else "user")

        if run_type == "xray":
            stats_cmd = "cd /usr/bin/xray && ./xray api stats --server=127.0.0.1:{} -name \"{}>>>{}>>>traffic>>>{}\""
            if is_reset == "true":
                stats_cmd = stats_cmd + " -reset"
        else:
            stats_cmd = "cd /usr/bin/v2ray && ./v2ctl api --server=127.0.0.1:{} StatsService.GetStats 'name: \"{}>>>{}>>>traffic>>>{}\"" + " reset: {}'".format(is_reset)

        stats_real_cmd = stats_cmd.format(str(self.door_port), type_tag, meta_info, "downlink")
        self.downlink_value = self.__run_command(stats_real_cmd)

        stats_real_cmd = stats_cmd.format(str(self.door_port), type_tag, meta_info, "uplink")
        self.uplink_value = self.__run_command(stats_real_cmd)

    def print_stats(self, horizontal=False):
        if horizontal:
            print("downlink: {0} uplink: {1} total: {2}".format(
                ColorStr.cyan(bytes_2_human_readable(self.downlink_value, 2)),
                ColorStr.cyan(bytes_2_human_readable(self.uplink_value, 2)),
                ColorStr.cyan(bytes_2_human_readable(self.downlink_value + self.uplink_value, 2)))
            )
        else:
            print('''
downlink: {0}  
uplink: {1} 
total: {2}
            '''.format(ColorStr.cyan(bytes_2_human_readable(self.downlink_value, 2)),
            ColorStr.cyan(bytes_2_human_readable(self.uplink_value, 2)),
            ColorStr.cyan(bytes_2_human_readable(self.downlink_value + self.uplink_value, 2)))
            )

def manage():

    FIND_V2RAY_CRONTAB_CMD = "crontab -l|grep {}".format(run_type)

    DEL_UPDATE_TIMER_CMD = "crontab -l|sed '/SHELL=/d;/{}/d' > crontab.txt && crontab crontab.txt >/dev/null 2>&1 && rm -f crontab.txt >/dev/null 2>&1".format(run_type)

    while True:
        loader = Loader()

        profile = loader.profile

        group_list = profile.group_list

        print("{}: {}".format(_("{} Traffic Statistics Status".format(run_type.capitalize())), profile.stats.status))

        print("")
        print(_("1.open statistics"))
        print("")
        print(_("2.close statistics"))
        print("")
        print(_("3.check user statistics result"))
        print("")
        print(_("4.check group statistics result"))
        print("")
        print(_("5.reset statistics"))
        print("")
        print(_("tip: restart {} will reset traffic statistics!!!".format(run_type)))
        print("")

        choice = readchar(_("please select: "))

        if choice in ("3", "4", "5") and not profile.stats.status:
            print(_("only open traffic statistics to operate"))
            print("")
            continue

        if choice == "1":
            if os.popen(FIND_V2RAY_CRONTAB_CMD).readlines():
                rchoice = readchar(_("open traffic statistics will close schedule update {}, continue?(y/n): ".format(run_type)))
                if rchoice == "y" or rchoice == "Y":
                    #关闭定时更新v2ray服务
                    os.system(DEL_UPDATE_TIMER_CMD)
                else:
                    print(_("undo open traffic statistics!!"))
                    continue
            gw = GlobalWriter(group_list)
            gw.write_stats(True)
            V2ray.restart()
            print(_("open traffic statistics success!"))
            print("")
            
        elif choice == "2":
            gw = GlobalWriter(group_list)
            gw.write_stats(False)
            V2ray.restart()
            print(_("close traffic statistics success!"))
            print("")

        elif choice == "3":
            sf = StatsFactory(profile.stats.door_port)
            print("")
            for group in group_list:
                port_way = "-{}".format(group.end_port) if group.end_port else ""
                for node in group.node_list:
                    print('''
Group: {group.tag}
IP: {color_ip}
Port: {group.port}{port_way}
{node}             
                    '''.format(group=group, color_ip=ColorStr.fuchsia(group.ip), node=node, port_way=port_way).strip())
                    if node.user_info:
                        sf.get_stats(node.user_info, False)
                        sf.print_stats(horizontal=True)
                    else:
                        print(ColorStr.yellow(_("no effective email!!!")))
                    print("")

        elif choice == "4":
            sf = StatsFactory(profile.stats.door_port)
            print("")
            for group in group_list:
                tls = _("open") if group.tls == "tls" else _("close")
                port_way = "-{}".format(group.end_port) if group.end_port else ""
                print('''
Group: {group.tag}
IP: {color_ip}
Port: {group.port}{port_way}
TLS: {tls}
                '''.format(group=group, color_ip=ColorStr.fuchsia(group.ip), tls=tls, port_way=port_way).strip())
                sf.get_stats(group.tag, False, True)
                sf.print_stats(horizontal=True)
                print("")

        elif choice == "5":
            if group_list[-1].node_list[-1].user_number > 1:
                print(profile)
                schoice = input(_("please input number or group alphabet to reset: "))
            else:
                schoice = "A"

            sf = StatsFactory(profile.stats.door_port)
            schoice=schoice.upper()
            if schoice.isnumeric() :
                schoice = int(schoice)
                if schoice > 0 and schoice <= group_list[-1].node_list[-1].user_number:
                    find = False
                    for group in group_list:
                        if find:
                            break
                        for node in group.node_list:
                            if node.user_number == schoice:
                                if node.user_info:
                                   sf.get_stats(node.user_info, True)
                                   sf.print_stats()
                                else:
                                    print(_("no effective email!!!"))
                                    print("")
                                find = True
                                break
            elif schoice.isalpha() and schoice <= group_list[-1].tag:
                sf.get_stats(schoice, True, True)
                sf.print_stats()
            else:
                print(_("input error, please input again"))
                print("")
        else:
            break


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import re
import sys
import tty
import socket
import string
import random
import termios
import pkg_resources
import urllib.request
from enum import Enum, unique

class ColorStr:
    RED = '\033[31m'       # 红色
    GREEN = '\033[32m'     # 绿色
    YELLOW = '\033[33m'    # 黄色
    BLUE = '\033[34m'      # 蓝色
    FUCHSIA = '\033[35m'   # 紫红色
    CYAN = '\033[36m'      # 青蓝色
    WHITE = '\033[37m'     # 白色
    #: no color
    RESET = '\033[0m'      # 终端默认颜色

    @classmethod
    def red(cls, s):
        return cls.RED + s + cls.RESET

    @classmethod
    def green(cls, s):
        return cls.GREEN + s + cls.RESET

    @classmethod
    def yellow(cls, s):
        return cls.YELLOW + s + cls.RESET

    @classmethod
    def blue(cls, s):
        return cls.BLUE + s + cls.RESET

    @classmethod
    def cyan(cls, s):
        return cls.CYAN + s + cls.RESET

    @classmethod
    def fuchsia(cls, s):
        return cls.FUCHSIA + s + cls.RESET

    @classmethod
    def white(cls, s):
        return cls.WHITE + s + cls.RESET

@unique
class StreamType(Enum):
    TCP = 'tcp'
    TCP_HOST = 'tcp_host'
    SOCKS = 'socks'
    SS = 'ss'
    MTPROTO = 'mtproto'
    H2 = 'h2'
    WS = 'ws'
    QUIC = 'quic'
    KCP = 'kcp'
    KCP_UTP = 'utp'
    KCP_SRTP = 'srtp'
    KCP_DTLS = 'dtls'
    KCP_WECHAT = 'wechat'
    KCP_WG = 'wireguard'
    VLESS_KCP = 'vless_kcp'
    VLESS_UTP = 'vless_utp'
    VLESS_SRTP = 'vless_srtp'
    VLESS_DTLS = 'vless_dtls'
    VLESS_WECHAT = 'vless_wechat'
    VLESS_WG = 'vless_wireguard'
    VLESS_TCP = 'vless_tcp'
    VLESS_TLS = 'vless_tls'
    VLESS_WS = 'vless_ws'
    VLESS_GRPC = 'vless_grpc'
    VLESS_XTLS = 'vless_xtls'
    TROJAN = 'trojan'

def header_type_list():
    return ("none", "srtp", "utp", "wechat-video", "dtls", "wireguard")

def ss_method():
    return ("aes-256-gcm", "aes-128-gcm", "chacha20-poly1305")

def xtls_flow():
    return ("", "xtls-rprx-origin", "xtls-rprx-direct")

def get_ip():
    """
    获取本地ip
    """
    my_ip = ""
    try:
        my_ip = urllib.request.urlopen('http://api.ipify.org').read()
    except Exception:
        my_ip = urllib.request.urlopen('http://icanhazip.com').read()
    return bytes.decode(my_ip).strip()

def port_is_use(port):
    """
    判断端口是否占用
    """
    tcp_use, udp_use = False, False
    s = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
    u = socket.socket(socket.AF_INET,socket.SOCK_DGRAM)
    s.settimeout(3)
    tcp_use = s.connect_ex(('127.0.0.1', int(port))) == 0
    try:
        u.bind(('127.0.0.1', int(port)))
    except:
        udp_use = True
    finally:
        u.close()
    return tcp_use or udp_use

def random_port(start_port, end_port):
    while True:
        random_port = random.randint(start_port, end_port)
        if not port_is_use(random_port):
            return random_port

def is_email(email):
    """
    判断是否是邮箱格式
    """
    str = r'^[a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+){0,4}@[a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+){0,4}$'
    return re.match(str, email)

def is_ipv4(ip):
    try:
        socket.inet_pton(socket.AF_INET, ip)
    except AttributeError:  # no inet_pton here, sorry
        try:
            socket.inet_aton(ip)
        except socket.error:
            return False
        return ip.count('.') == 3
    except socket.error:  # not a valid ip
        return False
    return True
 
def is_ipv6(ip):
    try:
        socket.inet_pton(socket.AF_INET6, ip)
    except socket.error:  # not a valid ip
        return False
    return True
 
def check_ip(ip):
    return is_ipv4(ip) or is_ipv6(ip)

def bytes_2_human_readable(number_of_bytes, precision=1):
    """
    流量bytes转换为kb, mb, gb等单位
    """
    if number_of_bytes < 0:
        raise ValueError("!!! number_of_bytes can't be smaller than 0 !!!")
 
    step_to_greater_unit = 1024.
 
    number_of_bytes = float(number_of_bytes)
    unit = 'bytes'
 
    if (number_of_bytes / step_to_greater_unit) >= 1:
        number_of_bytes /= step_to_greater_unit
        unit = 'KB'
 
    if (number_of_bytes / step_to_greater_unit) >= 1:
        number_of_bytes /= step_to_greater_unit
        unit = 'MB'
 
    if (number_of_bytes / step_to_greater_unit) >= 1:
        number_of_bytes /= step_to_greater_unit
        unit = 'GB'
 
    if (number_of_bytes / step_to_greater_unit) >= 1:
        number_of_bytes /= step_to_greater_unit
        unit = 'TB'
 
    number_of_bytes = round(number_of_bytes, precision)
 
    return str(number_of_bytes) + ' ' + unit

def gen_cert(domain):
    local_ip = get_ip()
    service_name = ["nginx", "httpd", "apache2"]
    start_cmd = "systemctl start {}  >/dev/null 2>&1"
    stop_cmd = "systemctl stop {} >/dev/null 2>&1"

    if not os.path.exists("/root/.acme.sh/acme.sh"):
        if ":" in local_ip:
            if not os.path.exists("/root/.acme.sh/"):
                os.makedirs("/root/.acme.sh")
            os.system("curl https://acme-install.netlify.app/acme.sh -o /root/.acme.sh/acme.sh")
        else:
            os.system("curl https://get.acme.sh | sh")

    open_port(80)
    if int(os.popen("/root/.acme.sh/acme.sh -v|tr -cd '[0-9]'").read()) >= 300:
        os.system("/root/.acme.sh/acme.sh --set-default-ca --server letsencrypt")
    get_ssl_cmd = "bash /root/.acme.sh/acme.sh --issue -d " + domain + " --debug --standalone --keylength ec-256"
    if ":" in local_ip:
        get_ssl_cmd = get_ssl_cmd + " --listen-v6"

    if not os.path.exists("/.dockerenv"):
        for name in service_name:
            os.system(stop_cmd.format(name))
    os.system(get_ssl_cmd)
    if not os.path.exists("/.dockerenv"):
        for name in service_name:
            os.system(start_cmd.format(name))

def calcul_iptables_traffic(port, ipv6=False):
    network = "1" if ipv6 else ""
    traffic_result = os.popen("bash {0} {1} {2}".format(pkg_resources.resource_filename("v2ray_util", "global_setting/calcul_traffic.sh"), str(port), network)).readlines()
    if traffic_result:
        traffic_list = traffic_result[0].split()
        upload_traffic = bytes_2_human_readable(int(traffic_list[0]), 2)
        download_traffic = bytes_2_human_readable(int(traffic_list[1]), 2)
        total_traffic = bytes_2_human_readable(int(traffic_list[2]), 2)
        return "{0}:  upload:{1} download:{2} total:{3}".format(ColorStr.green(str(port)), 
                ColorStr.cyan(upload_traffic), ColorStr.cyan(download_traffic), ColorStr.cyan(total_traffic))

def clean_iptables(port):
    import platform
    from .loader import Loader

    iptable_way = "iptables" if Loader().profile.network == "ipv4" else "ip6tables" 

    clean_cmd = "{} -D {} {}"
    check_cmd = "%s -nvL %s --line-number 2>/dev/null|grep -w \"%s\"|awk '{print $1}'|sort -r"
    firewall_clean_cmd = "firewall-cmd --zone=public --remove-port={}/tcp --remove-port={}/udp --permanent >/dev/null 2>&1"

    if "centos-8" in platform.platform():
        os.system("{}-save -c > /etc/sysconfig/iptables 2>/dev/null".format(iptable_way))
        os.system(firewall_clean_cmd.format(str(port), str(port)))
        os.system("firewall-cmd --reload >/dev/null 2>&1")
        os.system("{}-restore -c < /etc/sysconfig/iptables".format(iptable_way))
    input_result = os.popen(check_cmd % (iptable_way, "INPUT", str(port))).readlines()
    for line in input_result:
        os.system(clean_cmd.format(iptable_way, "INPUT", str(line)))

    output_result = os.popen(check_cmd % (iptable_way, "OUTPUT", str(port))).readlines()
    for line in output_result:
        os.system(clean_cmd.format(iptable_way, "OUTPUT", str(line)))

def all_port():
    from .loader import Loader
    profile = Loader().profile
    group_list = profile.group_list
    return set([group.port for group in group_list])

def iptables_open(iptable_way, port):
    check_cmd = "{} -nvL --line-number 2>/dev/null|grep -w \"{}\""
    input_cmd = "{} -I INPUT -p {} --dport {} -j ACCEPT"
    output_cmd = "{} -I OUTPUT -p {} --sport {}"
    if len(os.popen(check_cmd.format(iptable_way, port)).readlines()) > 0:
        return
    os.system(input_cmd.format(iptable_way, "tcp", port))
    os.system(input_cmd.format(iptable_way, "udp", port))
    os.system(output_cmd.format(iptable_way, "tcp", port))
    os.system(output_cmd.format(iptable_way, "udp", port))

def open_port(openport=-1):
    import platform
    from .loader import Loader

    iptable_way = "iptables" if Loader().profile.network == "ipv4" else "ip6tables"
    is_centos8 = True if "centos-8" in platform.platform() else False
    firewall_open_cmd = "firewall-cmd --zone=public --add-port={}/tcp --add-port={}/udp --permanent >/dev/null 2>&1"

    port_set = all_port()

    if openport != -1:
        port_str = str(openport)
        if is_centos8:
            os.system("{}-save -c > /etc/sysconfig/iptables 2>/dev/null".format(iptable_way))
            os.system(firewall_open_cmd.format(port_str, port_str))
            os.system("firewall-cmd --reload >/dev/null 2>&1")
            os.system("{}-restore -c < /etc/sysconfig/iptables".format(iptable_way))
        else:
            iptables_open(iptable_way, port_str)
    else:
        if is_centos8:
            os.system("{}-save -c > /etc/sysconfig/iptables 2>/dev/null".format(iptable_way))
            for port in port_set:
                os.system(firewall_open_cmd.format(str(port), str(port)))
            os.system("firewall-cmd --reload >/dev/null 2>&1") 
            os.system("{}-restore -c < /etc/sysconfig/iptables".format(iptable_way)) 
        for port in port_set:
            iptables_open(iptable_way, str(port))

def random_email():
    domain = ['163', 'qq', 'sina', '126', 'gmail', 'outlook', 'icloud']
    core_email = "@{}.com".format(random.choice(domain))
    return ''.join(random.sample(string.ascii_letters + string.digits, 8)) + core_email

def loop_input_choice_number(input_tip, length):
    """
    循环输入选择的序号,直到符合规定为止
    """
    while True:
        print("")
        if length >= 10:
            choice = input(input_tip)
        else:
            choice = readchar(input_tip)
        if not choice:
            return
        if choice.isnumeric():
            choice = int(choice)
        else:
            print(ColorStr.red(_("input error, please input again")))
            continue
        if (choice <= length and choice > 0):
            return choice
        else:
            print(ColorStr.red(_("input error, please input again")))

def readchar(prompt=""):
    if prompt:
        sys.stdout.write(prompt)
        sys.stdout.flush()
        
    fd = sys.stdin.fileno()
    old_settings = termios.tcgetattr(fd)
    try:
        tty.setraw(sys.stdin.fileno())
        ch = sys.stdin.read(1)
    finally:
        termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)

    print(ch)
    return ch.strip()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import json
import os

from v2ray_util import run_type
from .config import Config
from .utils import ColorStr, get_ip
from .group import SS, Socks, Vmess, Vless, Mtproto, Quic, Group, Dyport, Trojan

class Stats:
    def __init__(self, status=False, door_port=0):
        self.status = status
        self.door_port = door_port

    def __str__(self):
        return "open" if self.status else "close"

class Profile:
    def __init__(self):
        self.path = Config().get_path('config_path')
        self.group_list = []
        self.stats = None
        self.ban_bt = False
        self.user_number = 0
        self.network = "ipv4"
        self.modify_time = os.path.getmtime(self.path)
        self.read_json()

    def __str__(self):
        result = ""
        for group in self.group_list:
            result = "{}{}".format(result, group)
        result = result + _("Tip: The same group's node protocol, port, tls are the same.")
        return result

    def read_json(self):

        with open(self.path, 'r') as json_file:
            self.config = json.load(json_file)

        #读取配置文件大框架
        conf_inbounds = self.config["inbounds"]
        conf_rules = self.config["routing"]["rules"]

        stats = Stats()
        if "stats" in self.config:
            stats.status = True
            for inbound in conf_inbounds:
                if "protocol" in inbound and inbound["protocol"] == "dokodemo-door":
                    stats.door_port = inbound["port"]
                    break
        self.stats = stats

        for rule in conf_rules:
            if "protocol" in rule and "bittorrent" in rule["protocol"]:
                self.ban_bt = True

        local_ip = get_ip()

        if ":" in local_ip:
            self.network = "ipv6"

        group_ascii = 64  # before 'A' ascii code
        for index, json_part in enumerate(conf_inbounds):
            group = self.parse_group(json_part, index, local_ip)
            if group != None:
                group_ascii = group_ascii + 1
                if group_ascii > 90:
                    group.tag = str(group_ascii)
                else:
                    group.tag = chr(group_ascii)
                self.group_list.append(group)
        
        if len(self.group_list) == 0:
            print("{} json no streamSettings item, please run {} to recreate {} json!".format(run_type, ColorStr.cyan("{} new".format(run_type)), run_type))

        del self.config

    def parse_group(self, part_json, group_index, local_ip):
        dyp, quic, end_port, tfo, header, tls, path, host, conf_ip, serviceName, mode = Dyport(), None, None, None, "", "", "", "", local_ip, "", "gun"
        
        protocol = part_json["protocol"]

        if protocol == 'dokodemo-door' or (protocol == "vmess" and "streamSettings" not in part_json):
            return

        conf_settings = part_json["settings"]

        port_info = str(part_json["port"]).split("-", 2)

        if "domain" in part_json and part_json["domain"]:
            conf_ip = part_json["domain"]

        if len(port_info) == 2:
            port, end_port = port_info
        else:
            port = port_info[0]

        if "detour" in conf_settings:
            dynamic_port_tag = conf_settings["detour"]["to"]
            for inbound in self.config["inbounds"]:
                if "tag" in inbound and inbound["tag"] == dynamic_port_tag:
                    dyp.aid = inbound["settings"]["default"]["alterId"]
                    dyp.status = True
                    break

        if protocol in ("vmess", "vless", "socks", "trojan"):
            conf_stream = part_json["streamSettings"]
            tls = conf_stream["security"]

            if "sockopt" in conf_stream and "tcpFastOpen" in conf_stream["sockopt"]:
                tfo = "open" if conf_stream["sockopt"]["tcpFastOpen"] else "close"

            if "httpSettings" in conf_stream and conf_stream["httpSettings"]:
                path = conf_stream["httpSettings"]["path"]
            elif "wsSettings" in conf_stream and conf_stream["wsSettings"]:
                host = conf_stream["wsSettings"]["headers"]["Host"]
                path = conf_stream["wsSettings"]["path"]
            elif "tcpSettings" in conf_stream and conf_stream["tcpSettings"]:
                host = conf_stream["tcpSettings"]["header"]["request"]["headers"]["Host"]
                header = "http"

            if conf_stream["network"] == "kcp" and "header" in conf_stream["kcpSettings"]:
                header = conf_stream["kcpSettings"]["header"]["type"]
                if "seed" in conf_stream["kcpSettings"]:
                    path = conf_stream["kcpSettings"]["seed"]
            
            if conf_stream["network"] == "quic" and conf_stream["quicSettings"]:
                quic_settings = conf_stream["quicSettings"]
                quic = Quic(quic_settings["security"], quic_settings["key"], quic_settings["header"]["type"])
            if conf_stream["network"] == "grpc" and conf_stream["grpcSettings"]:
                serviceName = conf_stream["grpcSettings"]["serviceName"]
                if "multiMode" in conf_stream["grpcSettings"] and conf_stream["grpcSettings"]["multiMode"]:
                    mode = "multi"
        
        group = Group(conf_ip, port,  end_port=end_port, tls=tls, tfo=tfo, dyp=dyp, index=group_index)

        if protocol == "shadowsocks":
            self.user_number = self.user_number + 1
            email = conf_settings["email"] if 'email' in conf_settings else ''
            ss = SS(self.user_number, conf_settings["password"], conf_settings["method"], email)
            group.node_list.append(ss)
            group.protocol = ss.__class__.__name__
            return group
        elif protocol in ("vmess", "vless", "trojan"):
            clients=conf_settings["clients"]
        elif protocol == "socks":
            clients=conf_settings["accounts"]
        elif protocol == "mtproto":
            clients=conf_settings["users"]

        for client in clients:
            email, node, flow = "", None, ""
            self.user_number = self.user_number + 1
            if "email" in client and client["email"]:
                email = client["email"]

            if protocol == "vmess":
                node = Vmess(client["id"], client["alterId"], conf_stream["network"], self.user_number, path=path, host=host, header=header, email=email, quic=quic)

            elif protocol == "socks":
                node = Socks(self.user_number, client["pass"], user_info=client["user"])

            elif protocol == "mtproto":
                node = Mtproto(self.user_number, client["secret"], user_info=email)

            elif protocol == "vless":
                if tls == "xtls":
                    flow = client["flow"]
                node = Vless(client["id"], self.user_number, conf_settings["decryption"], email, conf_stream["network"], path, host, header, flow, serviceName, mode)

            elif protocol == "trojan":
                node = Trojan(self.user_number, client["password"], email)
                
            if not group.protocol:
                group.protocol = node.__class__.__name__

            group.node_list.append(node)
        return group


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import json
import base64
from urllib.parse import quote
from .utils import ColorStr

__author__ = 'Jrohy'

class Dyport:
    def __init__(self, status=False, aid=0):
        self.status = status
        self.aid = aid

    def __str__(self):
        return "{}, alterId:{}".format(_("open"), self.aid) if self.status else _("close")

class Quic:
    def __init__(self, security="none", key="", header="none"):
        self.security = security
        self.key = key
        self.header = header
    
    def __str__(self):
        return "Security: {0}\nKey: {1}\nHeader: {2}".format(self.security, self.key, self.header)

class User:
    def __init__(self, user_number, password, user_info=None):
        """
        user_info可能是email, 也可能user_name, 具体取决于group的protocol
        """
        self.__password = password
        self.user_info = user_info
        self.user_number = user_number

    @property
    def password(self):
        return self.__password

class SS(User):
    def __init__(self, user_number, password, method, user_info):
        super(SS, self).__init__(user_number, password, user_info)
        self.method = method
    
    def __str__(self):
        if self.user_info:
            return "Email: {self.user_info}\nMethod: {self.method}\nPassword: {password}\n".format(self=self, password=self.password)
        else:
            return "Method: {self.method}\nPassword: {password}\n".format(self=self, password=self.password)

    def link(self, ip, port, tls):
        ss_origin_url = "{0}:{1}@{2}:{3}".format(self.method, self.password, ip, port)
        return ColorStr.green("ss://{}".format(bytes.decode(base64.b64encode(bytes(ss_origin_url, 'utf-8')))))

    def stream(self):
        return "shadowsocks"

class Trojan(User):
    def __init__(self, user_number, password, email):
        super(Trojan, self).__init__(user_number, password, email)
    
    def __str__(self):
        if self.user_info:
            return "Email: {self.user_info}\nPassword: {password}\n".format(self=self, password=self.password)
        else:
            return "Password: {password}\n".format(password=self.password)

    def link(self, ip, port, tls):
        return ColorStr.green("trojan://{0}@{1}:{2}".format(self.password, ip, port))

    def stream(self):
        return "trojan"

class Mtproto(User):
    def __str__(self):
        if self.user_info:
            return "Email: {}\nSecret: {}\n".format(self.user_info, self.password)
        else:
            return "Secret: {}\n".format(self.password)
    
    def link(self, ip, port, tls):
        return ColorStr.green("tg://proxy?server={0}&port={1}&secret={2}".format(ip, port, self.password))

    def stream(self):
        return "mtproto"

class Socks(User):
    def __str__(self):
        return "User: {0}\nPass: {1}\nUDP: true\n".format(self.user_info, self.password)

    def link(self, ip, port, tls):
        if tls == "tls":
            return ColorStr.red(_("HTTPS Socks5 don't support telegram share link"))
        else:
            return ColorStr.green("tg://socks?server={0}&port={1}&user={2}&pass={3}".format(ip, port, self.user_info, self.password))

    def stream(self):
        return "socks"

class Vless(User):
    def __init__(self, uuid, user_number, encryption=None, email=None, network=None, path=None, host=None, header=None, flow="", serviceName="", mode=""):
        super(Vless, self).__init__(user_number, uuid, email)
        self.encryption = encryption
        self.path = path
        self.host = host
        self.header = header
        self.network = network
        self.flow = flow
        self.serviceName = serviceName
        self.mode = mode

    def __str__(self):
        email = ""
        if self.user_info:
            email = "Email: {}".format(self.user_info)
        result = '''
{email}
ID: {password}
Encryption: {self.encryption}
Network: {network}
'''.format(self=self, password=self.password, email=email, network=self.stream()).strip() + "\n"
        return result
    
    def stream(self):
        if self.network == "ws":
            return "WebSocket host: {0}, path: {1}".format(self.host, self.path)
        elif self.network == "tcp":
            return "tcp"
        elif self.network == "grpc":
            return "grpc serviceName: {}, mode: {}".format(self.serviceName, self.mode)
        elif self.network == "kcp":
            result = "kcp"
            if self.header and self.header != 'none':
                result = "{} {}".format(result, self.header)
            if self.path != "":
                result = "{} seed: {}".format(result, self.path)
            return result

    def link(self, ip, port, tls):
        result_link = "vless://{s.password}@{ip}:{port}?encryption={s.encryption}".format(s=self, ip=ip, port=port)
        if tls == "tls":
            result_link += "&security=tls"
        elif tls == "xtls":
            result_link += "&security=xtls&flow={}".format(self.flow)
        if self.network == "ws":
            result_link += "&type=ws&host={0}&path={1}".format(self.host, quote(self.path))
        elif self.network == "tcp":
            result_link += "&type=tcp"
        elif self.network == "grpc":
            result_link += "&type=grpc&serviceName={}&mode={}".format(self.serviceName, self.mode)
        elif self.network == "kcp":
            result_link += "&type=kcp&headerType={0}&seed={1}".format(self.header, self.path)
        return ColorStr.green(result_link)

class Vmess(User):
    def __init__(self, uuid, alter_id: int, network: str, user_number, *, path=None, host=None, header=None, email=None, quic=None):
        super(Vmess, self).__init__(user_number, uuid, email)
        self.alter_id = alter_id
        self.network = network
        self.path = path
        self.host = host
        self.header = header
        self.quic = quic
        if quic:
            self.header = quic.header
            self.host = quic.security
            self.path = quic.key

    def stream(self):
        network = ""
        if self.network == "quic":
            network = "Quic\n{}".format(self.quic)
        elif self.network == "h2":
            network = "HTTP/2 path: {}".format(self.path)
        elif self.network == "ws":
            network = "WebSocket host: {0}, path: {1}".format(self.host, self.path)
        elif self.network == "tcp":
            if self.host:
                network = "tcp host: {0}".format(self.host)
            else:
                network = "tcp"
        elif self.network == "kcp":
            network = "kcp"
            if self.header and self.header != 'none':
                network = "{} {}".format(network, self.header)
            if self.path != "":
                network = "{} seed: {}".format(network, self.path)
        return network

    def __str__(self):
        email = ""
        if self.user_info:
            email = "Email: {}".format(self.user_info)
        result = '''
{email}
UUID: {uuid}
Alter ID: {self.alter_id}
Network: {network}
'''.format(self=self, uuid=self.password, email=email, network=self.stream()).strip() + "\n"
        return result

    def link(self, ip, port, tls):
        json_dict = {
            "v": "2",
            "ps": "",
            "add": ip,
            "port": port,
            "aid": self.alter_id,
            "type": self.header,
            "net": self.network,
            "path": self.path,
            "host": self.host,
            "id": self.password,
            "tls": tls
        }
        json_data = json.dumps(json_dict)
        result_link = "vmess://{}".format(bytes.decode(base64.b64encode(bytes(json_data, 'utf-8'))))
        return ColorStr.green(result_link)

class Group:
    def __init__(self, ip, port, *, end_port=None, tfo=None, tls="none", dyp=Dyport(), index=0, tag='A'):
        self.ip = ip
        self.port = port
        self.end_port = end_port
        self.tag = tag
        self.node_list = []
        self.tfo = tfo
        self.tls = tls
        self.dyp = dyp
        self.protocol = None
        self.index = index

    def show_node(self, index):
        tfo = "TcpFastOpen: {}".format(self.tfo) if self.tfo != None else ""
        dyp = "DynamicPort: {}".format(self.dyp) if self.dyp.status else ""
        port_way = "-{}".format(self.end_port) if self.end_port else ""
        node = self.node_list[index]
        if self.tls == "tls":
            tls = _("open")
        elif self.tls == "xtls":
            tls = "xtls {0}, flow: {1}".format(_("open"), node.flow)
        else:
            tls = _("close")
        result = '''
{node.user_number}.
Group: {self.tag}
IP: {color_ip}
Port: {self.port}{port_way}
TLS: {tls}
{node}{tfo}
{dyp}'''.format(self=self, color_ip=ColorStr.fuchsia(self.ip), port_way=port_way, node=node,tfo=tfo, dyp=dyp,tls=tls)
        link = node.link(self.ip, int(self.port), self.tls)
        if link:
            result += "{}\n\n".format(link)
        return result

    # print一个实例打印的字符串
    def __str__(self):
        tfo = "TcpFastOpen: {}".format(self.tfo) if self.tfo != None else ""
        dyp = "DynamicPort: {}".format(self.dyp) if self.dyp.status else ""
        port_way = "-{}".format(self.end_port) if self.end_port else ""
        if self.tls == "tls":
            tls = _("open")
        elif self.tls == "xtls":
            tls = "xtls {0}, flow: {1}".format(_("open"), self.node_list[0].flow)
        else:
            tls = _("close")
        result = ""
        for node in self.node_list:
            temp = '''
{node.user_number}.
Group: {self.tag}
IP: {color_ip}
Port: {self.port}{port_way}
TLS: {tls}
{node}{tfo}
{dyp}
            '''.format(self=self, color_ip=ColorStr.fuchsia(self.ip), node=node,tfo=tfo,dyp=dyp,tls=tls, port_way=port_way)
            link = node.link(self.ip, int(self.port), self.tls)
            result = "{0}{1}\n\n".format(result, temp.strip())
            if link:
                result += "{}\n\n".format(link)             
        return result

    # 直接调用实例和打印一个实例显示的字符串一样
    def __repr__ (self):
        return self.__str__


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import random
import string

from ..util_core.group import SS
from ..util_core.v2ray import restart
from ..util_core.writer import GroupWriter
from ..util_core.selector import GroupSelector
from ..util_core.utils import ss_method, ColorStr, readchar

class SSFactory:
    def __init__(self):
        self.method_tuple = ss_method()

    def get_method(self):
        for index, method in enumerate(self.method_tuple):
            print ("{}.{}".format(index + 1, method))
        choice = readchar(_("please select shadowsocks method:"))
        if choice:
            choice = int(choice)
            if choice < 0 or choice > len(self.method_tuple):
                print(_("input out of range!!"))
                exit(-1)
            else:
                return self.method_tuple[choice - 1]
        else:
            exit(-1)   

    def get_password(self):
        random_pass = ''.join(random.sample(string.ascii_letters + string.digits, 16))
        new_pass =input("{} {}, {}".format(_("random generate password"), ColorStr.green(random_pass), _("enter to use, or input customize password: ")))
        if not new_pass:
            new_pass = random_pass
        return new_pass

@restart()
def modify(alter_type='method'):
    # 外部传参来决定修改哪种, 默认修改method
    correct_way = ("method", "password")

    if alter_type not in correct_way:
        print(_("input error!"))
        exit(-1)

    gs = GroupSelector(_('modify SS'))
    group = gs.group

    if group == None:
        exit(-1)
    elif type(group.node_list[0]) != SS:
        print("")
        print(_("local group not Shadowsocks protocol!"))
        print("")
        exit(-1)
    else:
        sm = SSFactory()
        gw = GroupWriter(group.tag, group.index)
        if alter_type == correct_way[0]:
            gw.write_ss_method(sm.get_method())
        elif alter_type == correct_way[1]:
            gw.write_ss_password(sm.get_password())
        print("{0} {1} {2}\n".format(_("modify Shadowsocks"),alter_type, _("success")))
        return True


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import random
import string

from v2ray_util import run_type
from ..util_core.v2ray import restart
from ..util_core.writer import StreamWriter, GroupWriter
from ..util_core.selector import GroupSelector, CommonSelector
from ..util_core.utils import StreamType, header_type_list, ColorStr, all_port, xtls_flow, readchar

from .ss import SSFactory

class StreamModifier:
    def __init__(self, group_tag='A', group_index=-1):
        self.stream_type = [
            (StreamType.TCP, "TCP"), 
            (StreamType.TCP_HOST, "Fake HTTP"), 
            (StreamType.WS, "WebSocket"), 
            (StreamType.KCP, "mKCP"), 
            (StreamType.KCP_SRTP, "mKCP + srtp"), 
            (StreamType.KCP_UTP, "mKCP + utp"), 
            (StreamType.KCP_WECHAT, "mKCP + wechat-video"),
            (StreamType.KCP_DTLS, "mKCP + dtls"), 
            (StreamType.KCP_WG, "mKCP + wireguard"), 
            (StreamType.H2, "HTTP/2"), 
            (StreamType.SOCKS, "Socks5"), 
            (StreamType.MTPROTO, "MTProto"), 
            (StreamType.SS, "Shadowsocks"),
            (StreamType.QUIC, "Quic"),
            (StreamType.VLESS_KCP, "VLESS + mkcp"),
            (StreamType.VLESS_UTP, "VLESS + mKCP + utp"),
            (StreamType.VLESS_SRTP, "VLESS + mKCP + srtp"),
            (StreamType.VLESS_WECHAT, "VLESS + mKCP + wechat-video"),
            (StreamType.VLESS_DTLS, "VLESS + mKCP + dtls"),
            (StreamType.VLESS_WG, "VLESS + mKCP + wireguard"),
            (StreamType.VLESS_TCP, "VLESS_TCP"),
            (StreamType.VLESS_TLS, "VLESS_TLS"),
            (StreamType.VLESS_WS, "VLESS_WS"),
            (StreamType.VLESS_XTLS, "VLESS_XTLS"),
            (StreamType.VLESS_GRPC, "VLESS_GRPC"),
            (StreamType.TROJAN, "Trojan"),
        ]
        self.group_tag = group_tag
        self.group_index = group_index

    def select(self, sType):
        sw = StreamWriter(self.group_tag, self.group_index, sType)
        kw = {}
        if sType in (StreamType.TCP_HOST, StreamType.WS):
            host = input(_("please input fake domain: "))
            kw['host'] = host
        elif sType == StreamType.SOCKS:
            user = input(_("please input socks user: "))
            password = input(_("please input socks password: "))
            if user == "" or password == "":
                print(_("socks user or password is null!!"))
                exit(-1)
            kw = {'user': user, 'pass': password}
        elif sType == StreamType.SS:
            sf = SSFactory()
            kw = {"method": sf.get_method(), "password": sf.get_password()}
        elif sType == StreamType.QUIC:
            key = ""
            security_list = ('none', "aes-128-gcm", "chacha20-poly1305")
            print("")
            security = CommonSelector(security_list, _("please select ss method: ")).select()
            if security != "none":
                key = ''.join(random.sample(string.ascii_letters + string.digits, 8))
                new_pass = input('{} {}, {}'.format(_("random generate password"), key, _("enter to use, or input customize password: ")))
                if new_pass:
                    key = new_pass
                    
            print("")
            header = CommonSelector(header_type_list(), _("please select fake header: ")).select()
            kw = {'security': security, 'key': key, 'header': header}
        elif sType in (StreamType.VLESS_TLS, StreamType.VLESS_WS, StreamType.VLESS_XTLS, StreamType.VLESS_GRPC):
            port_set = all_port()
            if not "443" in port_set:
                print()
                print(ColorStr.yellow(_("auto switch 443 port..")))
                gw = GroupWriter(self.group_tag, self.group_index)
                gw.write_port(443)
                sw = StreamWriter(self.group_tag, self.group_index, sType)
            if sType == StreamType.VLESS_WS:
                host = input(_("please input fake domain: "))
                kw['host'] = host
            elif sType == StreamType.VLESS_XTLS:
                flow_list = xtls_flow()
                print("")
                flow = CommonSelector(flow_list, _("please select xtls flow type: ")).select()
                kw = {'flow': flow}
            elif sType == StreamType.VLESS_GRPC and run_type == "xray":
                choice = readchar(_("open xray grpc multiMode?(y/n): ")).lower()
                if choice == 'y':
                    kw = {'mode': 'multi'}
                
        elif sType == StreamType.TROJAN:
            port_set = all_port()
            if not "443" in port_set:
                print()
                print(ColorStr.yellow(_("auto switch 443 port..")))
                gw = GroupWriter(self.group_tag, self.group_index)
                gw.write_port(443)
                sw = StreamWriter(self.group_tag, self.group_index, sType)
            random_pass = ''.join(random.sample(string.digits + string.ascii_letters, 8))
            tip = _("create random trojan user password:") + ColorStr.cyan(random_pass) + _(", enter to use or input new password: ")
            password = input(tip)
            if password == "":
                password = random_pass
            kw['password'] = password
        sw.write(**kw)

    def random_kcp(self):
        kcp_list = (StreamType.KCP_SRTP, StreamType.KCP_UTP, StreamType.KCP_WECHAT, StreamType.KCP_DTLS, StreamType.KCP_WG)
        choice = random.randint(0, 4)
        print("{}: {} \n".format(_("random generate (srtp | wechat-video | utp | dtls | wireguard) fake header, new protocol"), ColorStr.green(kcp_list[choice].value)))
        self.select(kcp_list[choice])

@restart()
def modify(group=None, sType=None):
    need_restart = False
    if group == None:
        need_restart = True
        gs = GroupSelector(_('modify protocol'))
        group = gs.group

    if group == None:
        pass
    else:
        sm = StreamModifier(group.tag, group.index)

        if sType != None:
            sm.select([v for v in StreamType if v.value == sType][0])
            print(_("modify protocol success"))
            return

        if need_restart:
            print("")
            print("{}: {}".format(_("group protocol"), group.node_list[0].stream()))

        print("")
        for index, stream_type in enumerate(sm.stream_type):
            print("{0}.{1}".format(index + 1, stream_type[1]))

        print("")
        choice = input(_("please select new protocol: "))

        if not choice.isdecimal():
            print(_("please input number!"))
        else:
            choice = int(choice)
            if choice > 0 and choice <= len(sm.stream_type):
                if sm.stream_type[choice - 1][1] in ("MTProto", "Shadowsocks") and group.tls in ('tls', 'xtls'):
                    print(_("{} MTProto/Shadowsocks not support https, close tls success!".format(run_type.capitalize())))
                sm.select(sm.stream_type[choice - 1][0])
                print(_("modify protocol success"))
                if need_restart:
                    return True